==> mcd-0.1.tar.gz: mcd-0.1/draw.c <==
/*
 * Author strongly advices against using this code, or a part of it,
 * in an application designed to run on any Microsoft(tm) platfrom.
 *
 * See doc/README for more information about COPYING terms.
 */
#include "mcd.h"

/* INTERN */

static void draw_hline(int i, char *ch); /* draw an hline */
static void draw_box(int x1, int y1, int x2, int y2); /* draw a box */
static void draw_tracks(struct mcd *cd);
static void draw_method(char *msg);

static void draw_hline(int i, char *ch) {
 if (i<0) return;
 while (i!=1) {
  show1(ch);--i;
 }
}

static void draw_box(int x1, int y1, int x2, int y2) {
 register int i;
 show1(C_BOX); /* color */
 gotoxy(x1,y1);show1(AC_TL);draw_hline(x2-x1,AC_HL);show1(AC_TR);
 for (i=1;i!=y2-1;++i) {
  gotoxy(x1,y1+i);show1(AC_VL);
  gotoxy(x2,y1+i);show1(AC_VL);
 }
 gotoxy(x1,y2);show1(AC_BL);draw_hline(x2-x1,AC_HL);show1(AC_BR);
}

static void draw_tracks(struct mcd *cd) {
 register int i=1,m=1; /* title + modifier */
 int y=0,x=0; /* x,y==diffs to real */

 if ((cd->title[1]=cd_readtracks(&cd[0]))<1) {
  draw_status("can't read tracks     ",1);return;
 }
 for (i=1;(i<=cd->title[1] && i<=99); i++) {
  m=((i-(pluskey+1)*10)<0 && (i-(pluskey+1)*10)>-11); /* is shortcut? */
  if (i%(maxy-10)==0) {y=-i;x+=15;} /* bottom reached, horiz. isn't checked! */
  gotoxy(3+x,10+i+y);
  if (m&&(i==cd->title[0])) _printf(C_STATUS "%02d" C_NORMAL, i); /* current */
  else if (m||(i==cd->title[0])) _printf(T_BOLD "%02d" C_NORMAL, i); /* current */
  else _printf("%02d",i); /* nothing special */

  if (i==cd->title[0]) show1(T_BOLD); /* current */
  _printf("%c - %02d:%02d\n",
   cd->t[i].audio?'d':'a',
   cd->t[i].min,
   cd->t[i].sec,(i-(pluskey+1)*10),m);
  if (i==cd->title[0]) show1(C_NORMAL); /* current */
 }
 gotoxy(13,10);
 _printf(C_STATUS "(%d)" C_NORMAL, pluskey);
}

static void draw_method(char *msg) {
 gotoxy(11,5);
 _printf(C_STATUS "%-20s" C_NORMAL, msg);
}


/* GLOBAL */

void draw_initscr(void) {
 draw_box(1,1,maxx,maxy);
 show1(C_BOX);
 gotoxyw(1,9, AC_ML);draw_hline(maxx-1,AC_HL);
 show1(AC_MR C_HEADLINE);
 gotoxyw(3,2, "mcd version " MCD_VERSION);
 gotoxyw(3,10,"tracklist");

 show1(C_DESCRIPT);
 gotoxyw(3,4, "Status:");
 gotoxyw(3,5, "Method:");
 gotoxyw(3,6, "Volume:");
 gotoxyw(3,7, "Track:");
 gotoxyw(3,8, "CD:");
 show1(C_NORMAL);

 /* draw_box(30,2,maxx-1,8); for titel etc.. reseved */
}

void draw_status(char *msg, short update) {
 gotoxy(11,4);
 _printf(C_STATUS "%-20s" C_NORMAL, msg);
 if (update) {
  updates[U_STATUS]=1;
  sleep(1);
 }
}

void draw_updatescr(struct mcd *cd) {
 if (cd_readsubchannel(cd)<0) {
  havecd=0;
  draw_status("no disc",0);
 }

 show1(C_NORMAL);
 if (updates[U_ALL]==1) {
  clear();
  tty_getsize(&maxx,&maxy);
  draw_initscr();
  updates[U_ALL]=0;
 }

 if (updates[U_TRACKS]==1) {
 
  //cd_start(&cd[0]);
  draw_tracks(cd);
  updates[U_TRACKS]=0;
 }

 if (updates[U_STATUS]==1) {
  draw_status(
  (cd->cdstat[0]==CDROM_AUDIO_NO_STATUS) ? "no status              ":"" &&
  (cd->cdstat[0]==CDROM_AUDIO_ERROR    ) ? "error                  ":"" &&
  (cd->cdstat[0]==CDROM_AUDIO_COMPLETED) ? "completed              ":"" &&
  (cd->cdstat[0]==CDROM_AUDIO_PAUSED   ) ? "paused                 ":"" &&
  (cd->cdstat[0]==CDROM_AUDIO_PLAY     ) ? "playing                ":"" &&
  (cd->cdstat[0]==CDROM_AUDIO_INVALID  ) ? "invalid                ":"",0);
 }

 if (updates[U_METHOD]==1) {
  draw_method(
  (cd->method[0]==M_PLAY_CD)    ? "play cd     ":"" &&
  (cd->method[0]==M_REPEAT_CD)  ? "repeat cd   ":"" &&
  (cd->method[0]==M_REPEAT_TRK) ? "repeat track":"" &&
  (cd->method[0]==M_SHUFFLE)    ? "shuffle     ":"" &&
  (cd->method[0]==M_INTRO)      ? "intro       ":"");
  updates[U_METHOD]=0;
 }

 show1(C_STATUS);
 if (updates[U_VOLUME]==1) {
  gotoxy(11,6);
  _printf("%d%%  ", (int)(cd->vol.channel0/2.55));
  updates[U_VOLUME]=0;
 }

 /* always */
 gotoxy(11,7);
 _printf("%02d:%02d (%d/%d)",
  cd->cds.cdsc_reladdr.msf.minute,
  cd->cds.cdsc_reladdr.msf.second,
  cd->title[0], cd->title[1]);
 gotoxy(11,8);
 _printf("%02d:%02d (%d/%d)",
  cd->cds.cdsc_absaddr.msf.minute,
  cd->cds.cdsc_absaddr.msf.second, 1, 1); /* maybe more then one cd l8er */
 gotoxy(maxx-1,maxy-1);
}

==> mcd-0.2a.tar.gz: mcd-0.2a/draw.c <==
/*
 * Author strongly advices against using this code, or a part of it,
 * in an application designed to run on any Microsoft(tm) platfrom.
 *
 * See doc/README for more information about COPYING terms.
 */
#include "mcd.h"

static void draw_hline(int i, char *ch); /* draw an hline */
static void draw_box(int x1, int y1, int x2, int y2); /* draw a box */
static void draw_method(char *msg);

static void draw_hline(int i, char *ch) {
 if (i<0) return;
 while (i!=1) {
  show1(ch);--i;
 }
 return;
}

static void draw_box(int x1, int y1, int x2, int y2) {
 register int i;
 show1(C_BOX); /* color */
 gotoxy(x1,y1);show1(AC_TL);draw_hline(x2-x1,AC_HL);show1(AC_TR);
 for (i=1;i!=y2-1;++i) {
  gotoxy(x1,y1+i);show1(AC_VL);
  gotoxy(x2,y1+i);show1(AC_VL);
 }
 gotoxy(x1,y2);show1(AC_BL);draw_hline(x2-x1,AC_HL);show1(AC_BR);
 return;
}

static void draw_method(char *msg) {
 gotoxy(11,5);
 _printf(C_STATUS "%s" C_NORMAL, msg);
 return;
}


/* GLOBAL */
short maxx,maxy;  /* screenmaxima */

void draw_tracks(struct mcd *cd) {
 register int i=1,m=1; /* title/modifier/realtitle */
 int y=0,x=0; /* x,y==diffs to real */

 for (i=1;(i<=cd->title[1] && i<=MCD_CDAUDIO_MAX); i++) {
  m=((i-(pluskey+1)*10)<0 && (i-(pluskey+1)*10)>-11); /* is shortcut? */
  if (i%(maxy-10)==0) {y=-i;x+=15;} /* bottom reached, horiz. isn't checked! */
  gotoxy(3+x,10+i+y);
  if (m&&(i==cd->title[0])) _printf(C_STATUS "%02d" C_NORMAL, i); /* current */
  else if (m||(i==cd->title[0])) _printf(T_BOLD "%02d" C_NORMAL, i); /* current */
  else _printf("%02d",i); /* nothing special */
  if (i==cd->title[0]) show1(T_BOLD); /* current */
  _printf("%c - %02d:%02d\n",
   cd->t[i].audio?'d':'a',
   cd->t[i].min,
   cd->t[i].sec);
  if (i==cd->title[0]) show1(C_NORMAL); /* current */
 }

 gotoxy(13,10);
 _printf(C_STATUS "(%d)" C_NORMAL, pluskey);
 return;
}

void draw_initscr(void) {
 draw_box(1,1,maxx,maxy);
 show1(C_BOX);
 gotoxyw(1,9, AC_ML);draw_hline(maxx-1,AC_HL);
 show1(AC_MR C_HEADLINE);
 gotoxyw(3,2, "mcd version " MCD_VERSION);
 gotoxyw(3,10,"tracklist");

 show1(C_DESCRIPT);
 gotoxyw(3,4, "Status:");
 gotoxyw(3,5, "Method:");
 gotoxyw(3,6, "Volume:");
 gotoxyw(3,7, "Track:");
 gotoxyw(3,8, "CD:");
 show1(C_NORMAL);

 /* draw_box(30,2,maxx-1,8); for titel etc.. reseved */
 return;
}

void draw_status(char *msg, short again) {
 gotoxy(11,4);
 _printf(C_STATUS "%-23s" C_NORMAL, msg);
 if (again) {
  updates[U_STATUS]=1;
  sleep(1);
 }
 return;
}

void draw_updatescr(struct mcd *cd) {
 if (cd_readsubchannel(cd)<0) {
  draw_status("no disc",0);
 }

 show1(C_NORMAL);
 if (updates[U_ALL]) {
  signal(SIGWINCH,tty_resize);
  clear();
  tty_getsize(&maxx,&maxy);
  if (maxx<42 || maxy<13) {
   show2("Sorry, you need at least a termsize of 42x13.\n");
   tty_mode(TERM_QUIT);
  }
  if ((cd->title[1]=cd_readtracks(&cd[0]))<1) {
   draw_status("can't read tracks",1);return;
  }
  draw_initscr();
  updates[U_TRACKS]=updates[U_STATUS]=updates[U_METHOD]=updates[U_VOLUME]=1;
  updates[U_ALL]=0;
 }

 if (updates[U_STATUS]) {
  draw_status(
  (cd->status==CDROM_AUDIO_NO_STATUS) ? "no status":"" &&
  (cd->status==CDROM_AUDIO_ERROR    ) ? "error    ":"" &&
  (cd->status==CDROM_AUDIO_COMPLETED) ? "stopped  ":"" &&
  (cd->status==CDROM_AUDIO_PAUSED   ) ? "paused   ":"" &&
  (cd->status==CDROM_AUDIO_PLAY     ) ? "playing  ":"" &&
  (cd->status==CDROM_AUDIO_INVALID  ) ? "invalid  ":"",0);
  updates[U_STATUS]=0;
 }

 if (updates[U_METHOD]) {
  draw_method(
  (cd->method==M_PLAY_CD)    ? "play cd     ":"" &&
  (cd->method==M_REPEAT_CD)  ? "repeat cd   ":"" &&
  (cd->method==M_REPEAT_TRK) ? "repeat track":"" &&
  (cd->method==M_INTRO)      ? "intro       ":"");
  updates[U_METHOD]=0;
 }

 if (updates[U_TRACKS]) {
  draw_tracks(&cd[0]);
  updates[U_TRACKS]=0;
 }

 show1(C_STATUS);
 if (updates[U_VOLUME]==1) {
  gotoxy(11,6);
  _printf("%d%%  ", (int)(cd->vol.channel0/2.55));
  updates[U_VOLUME]=0;
 }

 /* always */
 gotoxy(11,7);
 _printf("%02d:%02d (%2d/%2d)",
  cd->cds.cdsc_reladdr.msf.minute,
  cd->cds.cdsc_reladdr.msf.second,
  cd->title[0], cd->title[1]);
 gotoxy(11,8);
 _printf("%02d:%02d (%2d/%2d)",
  cd->cds.cdsc_absaddr.msf.minute,
  cd->cds.cdsc_absaddr.msf.second,
  1, 1); /* maybe more then one cd l8er */
  gotoxy(2,2);
 return;
}
